# -*- coding: UTF-8 -*-

import functools

import orjson
from psycopg.types.json import set_json_dumps, set_json_loads
from psycopg_pool import AsyncConnectionPool

set_json_loads(orjson.loads)
set_json_dumps(orjson.dumps)

_NAMED_POOL_DICT: dict[str, AsyncConnectionPool] = dict()


async def init(
        *, name: str, dsn: str, pool_size: int = 16,
) -> None:
    if name in _NAMED_POOL_DICT:
        raise ValueError(f'Pool "{name}" already exists')
    if pool_size < 1:
        raise ValueError('Pool size should be greater than 0')
    minsize, maxsize = pool_size, pool_size
    if minsize > 4:
        minsize = 4
    kwargs = dict(autocommit=False)
    pool = AsyncConnectionPool(
        conninfo=dsn, min_size=minsize, max_size=maxsize, open=False,
        name=name, kwargs=kwargs)
    await pool.open(wait=True)
    _NAMED_POOL_DICT[name] = pool


async def close_all():
    for name, pool in _NAMED_POOL_DICT.items():
        await pool.close()
    _NAMED_POOL_DICT.clear()


def with_postgres(*, name: str, transaction: bool = False):
    def wrapper(func):
        @functools.wraps(func)
        async def wrapped(*args, **kwargs):
            if name not in _NAMED_POOL_DICT:
                raise SyntaxError(f'Pool "{name}" not found')
            async with _NAMED_POOL_DICT[name].connection() as conn:
                async with conn.cursor() as cursor:
                    kwargs['cursor'] = cursor
                    if transaction:
                        async with conn.transaction():
                            result = await func(*args, **kwargs)
                    else:
                        result = await func(*args, **kwargs)
            return result
        return wrapped

    return wrapper



# -*- coding: UTF-8 -*-

import abc
from muso.request import MusoRequest


class AuthBase(abc.ABC):

    def __init__(self):
        self.uri = None
        self.method = None

    def fill_uri_and_method(self, *, uri: str, method: str):
        self.uri = uri
        self.method = method

    @property
    @abc.abstractmethod
    def auth_name(self):
        raise NotImplementedError

    @property
    @abc.abstractmethod
    def auth_type(self):
        raise NotImplementedError

    @abc.abstractmethod
    def __call__(self, *, request: MusoRequest):
        raise NotImplementedError



# -*- coding: UTF-8 -*-

import functools

from aiomysql import Connection, DictCursor, Pool, create_pool

_NAMED_POOL_DICT: dict[str, Pool] = dict()


async def init(
        *, name: str, charset: str = 'utf8mb4', pool_size: int = 16, **kwargs,
) -> None:
    if name in _NAMED_POOL_DICT:
        raise ValueError(f'Pool "{name}" already exists')
    if pool_size < 1:
        raise ValueError('Pool size should be greater than 0')
    minsize, maxsize = pool_size, pool_size
    if minsize > 4:
        minsize = 4
    _NAMED_POOL_DICT[name] = await create_pool(
        minsize=minsize, maxsize=maxsize, charset=charset, **kwargs)


async def close_all():
    for name, pool in _NAMED_POOL_DICT.items():
        pool.close()
        await pool.wait_closed()
    _NAMED_POOL_DICT.clear()


def with_mysql(*, name: str, transaction: bool = False,
               dict_cursor: bool = False):
    def decorator(f):
        @functools.wraps(f)
        async def wrapper(*args, **kwargs):
            pool = _NAMED_POOL_DICT.get(name)
            if not pool:
                raise SyntaxError(f'Pool {name} not found')
            async with pool.acquire() as connection:    # type: Connection
                if transaction:
                    await connection.begin()
                cursor_coro = (connection.cursor(DictCursor)
                               if dict_cursor else connection.cursor())
                try:
                    async with cursor_coro as cursor:
                        kwargs['cursor'] = cursor
                        result = await f(*args, **kwargs)
                    await connection.commit()
                except Exception as e:
                    await connection.rollback()
                    raise e
                return result

        return wrapper

    return decorator
